//
// PetAPI.swift
//
// Generated by openapi-generator
// https://openapi-generator.tech
//

import Foundation

public struct PetAPI {
    /**
     Add a new pet to the store
     - POST /pet
     - OAuth:
       - type: oauth2
       - name: petstore_auth
     - parameter body: (body) Pet object that needs to be added to the store 
     - returns: RequestBuilder<Void> 
     */
    public func addPet(body: Pet) -> RequestBuilder<Void> {
        let path = "/pet"

        var parameters: [Parameters] = []

        parameters.append(Parameters(body))

        return RequestBuilder<Void>(endpoint: path, method: "POST", parameters: parameters)
    }

    /**
     Deletes a pet
     - DELETE /pet/{petId}
     - OAuth:
       - type: oauth2
       - name: petstore_auth
     - parameter petId: (path) Pet id to delete 
     - parameter apiKey: (header)  (optional)
     - returns: RequestBuilder<Void> 
     */
    public func deletePet(petId: Int64, apiKey: String? = nil) -> RequestBuilder<Void> {
        var path = "/pet/{petId}"
        let petIdEscaped = "\(petId)".addingPercentEncoding(withAllowedCharacters: .urlPathAllowed) ?? ""
        path = path.replacingOccurrences(of: "{petId}", with: petIdEscaped, options: .literal, range: nil)

        return RequestBuilder<Void>(endpoint: path, method: "DELETE")
    }

    /**
     * enum for parameter status
     */
    public enum Status_findPetsByStatus: String {
        case available = "available"
        case pending = "pending"
        case sold = "sold"
    }

    /**
     Finds Pets by status
     - GET /pet/findByStatus
     - Multiple status values can be provided with comma separated strings
     - OAuth:
       - type: oauth2
       - name: petstore_auth
     - parameter status: (query) Status values that need to be considered for filter 
     - returns: RequestBuilder<[Pet]> 
     */
    public func findPetsByStatus(status: [String]) -> RequestBuilder<[Pet]> {
        let path = "/pet/findByStatus"

        var parameters: [Parameters] = []

        let queryParams: [String: Any?] = [
            "status": status
        ]
        parameters.append(Parameters(queryParams))
        return RequestBuilder<[Pet]>(endpoint: path, method: "GET", parameters: parameters)
    }

    /**
     Finds Pets by tags
     - GET /pet/findByTags
     - Multiple tags can be provided with comma separated strings. Use tag1, tag2, tag3 for testing.
     - OAuth:
       - type: oauth2
       - name: petstore_auth
     - parameter tags: (query) Tags to filter by 
     - returns: RequestBuilder<[Pet]> 
     */
    public func findPetsByTags(tags: [String]) -> RequestBuilder<[Pet]> {
        let path = "/pet/findByTags"

        var parameters: [Parameters] = []

        let queryParams: [String: Any?] = [
            "tags": tags
        ]
        parameters.append(Parameters(queryParams))
        return RequestBuilder<[Pet]>(endpoint: path, method: "GET", parameters: parameters)
    }

    /**
     Find pet by ID
     - GET /pet/{petId}
     - Returns a single pet
     - API Key:
       - type: apiKey api_key 
       - name: api_key
     - parameter petId: (path) ID of pet to return 
     - returns: RequestBuilder<Pet> 
     */
    public func getPetById(petId: Int64) -> RequestBuilder<Pet> {
        var path = "/pet/{petId}"
        let petIdEscaped = "\(petId)".addingPercentEncoding(withAllowedCharacters: .urlPathAllowed) ?? ""
        path = path.replacingOccurrences(of: "{petId}", with: petIdEscaped, options: .literal, range: nil)

        return RequestBuilder<Pet>(endpoint: path, method: "GET")
    }

    /**
     Update an existing pet
     - PUT /pet
     - OAuth:
       - type: oauth2
       - name: petstore_auth
     - parameter body: (body) Pet object that needs to be added to the store 
     - returns: RequestBuilder<Void> 
     */
    public func updatePet(body: Pet) -> RequestBuilder<Void> {
        let path = "/pet"

        var parameters: [Parameters] = []

        parameters.append(Parameters(body))

        return RequestBuilder<Void>(endpoint: path, method: "PUT", parameters: parameters)
    }

    /**
     Updates a pet in the store with form data
     - POST /pet/{petId}
     - OAuth:
       - type: oauth2
       - name: petstore_auth
     - parameter petId: (path) ID of pet that needs to be updated 
     - parameter name: (form) Updated name of the pet (optional)
     - parameter status: (form) Updated status of the pet (optional)
     - returns: RequestBuilder<Void> 
     */
    public func updatePetWithForm(petId: Int64, name: String? = nil, status: String? = nil) -> RequestBuilder<Void> {
        var path = "/pet/{petId}"
        let petIdEscaped = "\(petId)".addingPercentEncoding(withAllowedCharacters: .urlPathAllowed) ?? ""
        path = path.replacingOccurrences(of: "{petId}", with: petIdEscaped, options: .literal, range: nil)
        var parameters: [Parameters] = []

        let formParams: [String: String?] = [
            "name": name,
            "status": status
        ]
        parameters.append(Parameters(formParams))

        return RequestBuilder<Void>(endpoint: path, method: "POST", parameters: parameters)
    }

    /**
     uploads an image
     - POST /pet/{petId}/uploadImage
     - OAuth:
       - type: oauth2
       - name: petstore_auth
     - parameter petId: (path) ID of pet to update 
     - parameter additionalMetadata: (form) Additional data to pass to server (optional)
     - parameter file: (form) file to upload (optional)
     - returns: RequestBuilder<ApiResponse> 
     */
    public func uploadFile(petId: Int64, additionalMetadata: String? = nil, file: URL? = nil) -> RequestBuilder<ApiResponse> {
        var path = "/pet/{petId}/uploadImage"
        let petIdEscaped = "\(petId)".addingPercentEncoding(withAllowedCharacters: .urlPathAllowed) ?? ""
        path = path.replacingOccurrences(of: "{petId}", with: petIdEscaped, options: .literal, range: nil)
        var parameters: [Parameters] = []

        let formParams: [String: String?] = [
            "additionalMetadata": additionalMetadata,
            "file": file?.description
        ]
        parameters.append(Parameters(formParams))

        return RequestBuilder<ApiResponse>(endpoint: path, method: "POST", parameters: parameters)
    }

    /**
     uploads an image (required)
     - POST /fake/{petId}/uploadImageWithRequiredFile
     - OAuth:
       - type: oauth2
       - name: petstore_auth
     - parameter petId: (path) ID of pet to update 
     - parameter requiredFile: (form) file to upload 
     - parameter additionalMetadata: (form) Additional data to pass to server (optional)
     - returns: RequestBuilder<ApiResponse> 
     */
    public func uploadFileWithRequiredFile(petId: Int64, requiredFile: URL, additionalMetadata: String? = nil) -> RequestBuilder<ApiResponse> {
        var path = "/fake/{petId}/uploadImageWithRequiredFile"
        let petIdEscaped = "\(petId)".addingPercentEncoding(withAllowedCharacters: .urlPathAllowed) ?? ""
        path = path.replacingOccurrences(of: "{petId}", with: petIdEscaped, options: .literal, range: nil)
        var parameters: [Parameters] = []

        let formParams: [String: String?] = [
            "additionalMetadata": additionalMetadata,
            "requiredFile": requiredFile.description
        ]
        parameters.append(Parameters(formParams))

        return RequestBuilder<ApiResponse>(endpoint: path, method: "POST", parameters: parameters)
    }

}
